"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const apollo_server_express_1 = require("apollo-server-express");
const debug_1 = __importDefault(require("debug"));
const dotenv_1 = __importDefault(require("dotenv"));
const express_1 = __importDefault(require("express"));
const path_1 = __importDefault(require("path"));
const type_graphql_1 = require("type-graphql"); // formatArgumentValidationError
const typedi_1 = require("typedi");
const typeorm_1 = require("typeorm");
const logger_1 = require("./logger");
const gql_1 = require("../gql");
const middleware_1 = require("../middleware");
const sgql_1 = require("../sgql");
const sorm_1 = require("../sorm");
const code_generator_1 = require("./code-generator");
const openBrowser_1 = __importDefault(require("../utils/dev/openBrowser"));
const express_jwt_1 = __importDefault(require("express-jwt"));
const jsonwebtoken_1 = require("jsonwebtoken");
const getMetadataStorage_1 = require("type-graphql/dist/metadata/getMetadataStorage");
const debug = debug_1.default('tcg:server');
class Server {
    constructor(appOptions, dbOptions = {}) {
        this.appOptions = appOptions;
        this.dbOptions = dbOptions;
        this.mockDBConnection = false;
        this.introspection = true;
        // if (!process.env.NODE_ENV) {
        //   throw new Error("NODE_ENV must be set - use 'development' locally");
        // }
        dotenv_1.default.config();
        // Ensure that TypeORM and TypeGraphQL are all using the same typedi container
        this.container = this.appOptions.container || typedi_1.Container;
        typeorm_1.useContainer(this.container);
        const host = this.appOptions.host || process.env.APP_HOST;
        if (!host) {
            throw new Error('`host` is required');
        }
        this.appProtocol =
            (this.appOptions.protocol || process.env.APP_PROTOCOL) === 'https' ? 'https' : 'http';
        this.appHost = host;
        this.appToken = this.appOptions.token || process.env.APP_TOKEN || 'tcg@crud/secret-token';
        this.appPath = this.appOptions.path || process.env.APP_PATH || 'graphql';
        this.appPort = parseInt(String(this.appOptions.port || process.env.APP_PORT), 10) || 4000;
        this.authChecker = this.appOptions.authChecker || sgql_1.authChecker;
        this.bodyParserConfig = this.appOptions.bodyParserConfig;
        this.generatedFolder = this.appOptions.generatedFolder || path_1.default.join(process.cwd(), 'generated');
        typedi_1.Container.set('tcg.generated-folder', this.generatedFolder);
        this.logger = this.getLogger();
        typedi_1.Container.set('tcg.logger', this.logger); // Save for later so we can pull globally
        this.autoGenerateFiles =
            typeof this.appOptions.autoGenerateFiles !== 'undefined'
                ? this.appOptions.autoGenerateFiles
                : process.env.NODE_ENV === 'development';
        this.resolversPath = this.appOptions.resolversPath || [process.cwd() + '/**/*.resolver.ts'];
        this.introspection = !!this.appOptions.introspection;
    }
    getLogger() {
        if (this.appOptions.logger) {
            return this.appOptions.logger;
        }
        else if (typedi_1.Container.has('tcg.logger')) {
            return typedi_1.Container.get('tcg.logger');
        }
        return logger_1.logger;
    }
    async establishDBConnection() {
        if (!this.connection) {
            debug('establishDBConnection:start');
            const connectionFn = this.appOptions.mockDBConnection ? sorm_1.mockDBConnection : sorm_1.createDBConnection;
            this.connection = await connectionFn(this.dbOptions);
            debug('establishDBConnection:end');
        }
        return this.connection;
    }
    async getBinding(options = {}) {
        let binding;
        try {
            binding = await gql_1.getRemoteBinding(`${this.appProtocol}://${this.appHost}:${this.appPort}/${this.appPath}`, {
                origin: 'tcg',
                ...options
            });
            return binding;
        }
        catch (error) {
            if (error.result && error.result.errors) {
                const messages = error.result.errors.map((item) => item.message);
                throw new Error(JSON.stringify(messages));
            }
            throw error;
        }
    }
    async buildGraphQLSchema() {
        if (!this.schema) {
            debug('buildGraphQLSchema:start');
            this.schema = await type_graphql_1.buildSchema({
                authChecker: this.authChecker,
                container: this.container,
                globalMiddlewares: [middleware_1.DataLoaderMiddleware, ...(this.appOptions.middlewares || [])],
                resolvers: this.resolversPath
            });
            debug('buildGraphQLSchema:end');
        }
        return this.schema;
    }
    async generateFiles() {
        debug('start:generateFiles:start');
        getMetadataStorage_1.getMetadataStorage().clear();
        await this.establishDBConnection();
        await new code_generator_1.CodeGenerator(this.connection, this.generatedFolder, {
            resolversPath: this.resolversPath,
            tcgImportPath: this.appOptions.tcgImportPath
        }).generate();
        debug('start:generateFiles:end');
    }
    async start() {
        debug('start:start');
        await this.establishDBConnection();
        if (this.autoGenerateFiles) {
            await this.generateFiles();
        }
        await this.buildGraphQLSchema();
        const contextGetter = this.appOptions.context ||
            (async () => {
                return {};
            });
        debug('start:ApolloServerAllocation:start');
        this.graphQLServer = new apollo_server_express_1.ApolloServer({
            context: async (options) => {
                const consumerCtx = await contextGetter(options.req);
                return {
                    connection: this.connection,
                    dataLoader: {
                        initialized: false,
                        loaders: {}
                    },
                    request: options.req,
                    ...consumerCtx
                };
            },
            introspection: this.introspection,
            schema: this.schema,
            playground: { version: 'latest' }
            // tracing: true
        });
        debug('start:ApolloServerAllocation:end');
        const app = express_1.default();
        app.use('/health', middleware_1.healthCheckMiddleware);
        app.post(`/${this.appPath}`, express_jwt_1.default({ secret: this.appToken }));
        app.use((err, _req, res, _next) => {
            if (err.name === 'UnauthorizedError') {
                res.status(401).json({ data: null, errors: [{ message: `Invalid token` }] });
            }
        });
        debug('start:applyMiddleware:start');
        this.graphQLServer.applyMiddleware({
            app,
            path: `/${this.appPath}`,
            bodyParserConfig: this.bodyParserConfig
        });
        debug('start:applyMiddleware:end');
        const url = `${this.appProtocol}://${this.appHost}:${this.appPort}${this.graphQLServer.graphqlPath}`;
        debug(`url: ${url}`);
        this.httpServer = app.listen({ port: this.appPort }, () => {
            this.logger.info(`🚀 Server ready at ${url}`);
            this.logger.info(`🔑 Token: ${jsonwebtoken_1.sign({}, this.appToken, { noTimestamp: true })}`);
        });
        if (this.shouldOpenPlayground()) {
            openBrowser_1.default(url);
        }
        debug('start:end');
        return this;
    }
    async stop() {
        this.logger.info('Stopping HTTP Server');
        this.httpServer.close();
        this.logger.info('Closing DB Connection');
        await this.connection.close();
    }
    shouldOpenPlayground() {
        if (typeof this.appOptions.openPlayground !== 'undefined') {
            return Boolean(this.appOptions.openPlayground);
        }
        if (typeof process.env.JEST_WORKER_ID !== 'undefined') {
            return false;
        }
        return process.env.NODE_ENV === 'development';
    }
}
exports.Server = Server;
