"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const graphql_1 = require("graphql");
const utils_1 = require("./utils");
function entityToCreateInputImporter(entity) {
    return `
    import {${entity.name}CreateInput} from './input/create/${entity.name}CreateInput';
    export {${entity.name}CreateInput} from './input/create/${entity.name}CreateInput';
  `;
}
exports.entityToCreateInputImporter = entityToCreateInputImporter;
function entityToCreateInputsImporter(entities) {
    const createInputsNames = [];
    entities.forEach((entity) => {
        entity.relations.forEach((relation) => {
            const { graphqlType } = utils_1.relationToTypes(relation);
            if (!relation.inverseEntityMetadata || graphqlType.constructor !== graphql_1.GraphQLObjectType)
                return;
            const { inverseSidePropertyPath } = relation;
            const isInverse = Boolean(relation.inverseEntityMetadata.relations.find(inverseRelation => inverseRelation.propertyName === inverseSidePropertyPath));
            const inverseSidePropertyPathCapitalized = inverseSidePropertyPath.charAt(0).toUpperCase() + inverseSidePropertyPath.slice(1);
            const inputName = (prefix) => `${graphqlType.name}Create${prefix}${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`;
            const createInputName = inputName('');
            const createInputOneName = inputName('One');
            const createInputManyName = inputName('Many');
            if (!createInputsNames.includes(createInputName) && isInverse) {
                createInputsNames.push(createInputName);
            }
            switch (relation.relationType) {
                case 'many-to-one':
                case 'one-to-one':
                    if (createInputsNames.includes(createInputOneName))
                        break;
                    createInputsNames.push(createInputOneName);
                    break;
                case 'one-to-many':
                case 'many-to-many':
                    if (createInputsNames.includes(createInputManyName))
                        break;
                    createInputsNames.push(createInputManyName);
                    break;
            }
        });
    });
    return createInputsNames.reduce((prev, next) => `
     ${prev}
     import {${next}} from './input/create/${next}';
     export {${next}} from './input/create/${next}';
    `, '');
}
exports.entityToCreateInputsImporter = entityToCreateInputsImporter;
function entityToCreateInput(entity) {
    let fieldTemplates = '';
    const imports = [];
    const enumImports = [];
    entity.columns
        .filter((column) => !column.relationMetadata &&
        !column.isPrimary &&
        !column.isCreateDate &&
        !column.isGenerated &&
        !column.isUpdateDate &&
        !column.isVersion)
        .forEach((column) => {
        const graphQLDataType = utils_1.columnTypeToGraphQLDataType(column);
        const nullable = column.isNullable || column.default ? '{ nullable: true }' : '';
        const tsRequired = column.isNullable || column.default ? '?' : '!';
        const tsType = utils_1.columnToTypeScriptType(column);
        if (column.enum && !enumImports.includes(tsType)) {
            enumImports.push(tsType);
        }
        fieldTemplates += `
          @FieldGraphQL(() => ${graphQLDataType}, ${nullable})
          ${column.propertyName}${tsRequired}: ${tsType};
       `;
    });
    entity.relations.forEach((relation) => {
        const { graphqlType } = utils_1.relationToTypes(relation);
        if (!relation.inverseEntityMetadata || graphqlType.constructor !== graphql_1.GraphQLObjectType)
            return;
        const { propertyName, inverseSidePropertyPath } = relation;
        const isInverse = Boolean(relation.inverseEntityMetadata.relations.find(inverseRelation => inverseRelation.propertyName === inverseSidePropertyPath));
        const inverseSidePropertyPathCapitalized = inverseSidePropertyPath.charAt(0).toUpperCase() + inverseSidePropertyPath.slice(1);
        const nullable = relation.isNullable ? '{nullable:true}' : '';
        const tsRequired = relation.isNullable ? '?' : '!';
        switch (relation.relationType) {
            case 'many-to-one':
            case 'one-to-one':
                if (!imports.includes(`${graphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`)) {
                    imports.push(`${graphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`);
                }
                fieldTemplates += `
      @FieldGraphQL(() => ${graphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input,${nullable})
      ${propertyName}${tsRequired}: ${graphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input;
      `;
                break;
            case 'one-to-many':
            case 'many-to-many':
                if (!imports.includes(`${graphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`)) {
                    imports.push(`${graphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`);
                }
                fieldTemplates += `
      @FieldGraphQL(() => ${graphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input,${nullable})
      ${propertyName}${tsRequired}: ${graphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input;
      `;
                break;
        }
    });
    return `
    import 'reflect-metadata';
    import { ArgsType, Field as FieldGraphQL, Float, GraphQLISODateTime,
               ID, InputType,
               ObjectType, registerEnumType } from 'type-graphql';
    import { GraphQLJSONObject, IDType, ConnectionArgs, PageInfo,
             GraphQLBoolean, GraphQLFloat, GraphQLID,
             GraphQLInt, GraphQLString, ${enumImports.join(',')} } from '../..';
    ${imports.length
        ? `
    import { ${imports.join(', ')} } from '../..';
    `
        : ''}
    @InputType()
    export class ${entity.name}CreateInput {
      ${fieldTemplates}
    }
  `;
}
exports.entityToCreateInput = entityToCreateInput;
function entityToCreateInputs(entity) {
    const createInputs = {};
    entity.relations.forEach((relation) => {
        const { graphqlType } = utils_1.relationToTypes(relation);
        if (!relation.inverseEntityMetadata || graphqlType.constructor !== graphql_1.GraphQLObjectType)
            return;
        const { inverseSidePropertyPath } = relation;
        const isInverse = !!relation.inverseEntityMetadata.relations.find(inverseRelation => inverseRelation.propertyName === inverseSidePropertyPath);
        const inverseSidePropertyPathCapitalized = inverseSidePropertyPath.charAt(0).toUpperCase() + inverseSidePropertyPath.slice(1);
        const inputName = (prefix) => `${graphqlType.name}Create${prefix}${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`;
        const getFields = () => ({
            fields: relation.inverseEntityMetadata.columns
                .filter((column) => !(column.relationMetadata || column.propertyName === inverseSidePropertyPath) &&
                !column.isPrimary &&
                !column.isCreateDate &&
                !column.isGenerated &&
                !column.isUpdateDate &&
                !column.isVersion)
                .map((column) => {
                const graphQLDataType = utils_1.columnTypeToGraphQLDataType(column);
                const nullable = column.isNullable || column.default ? '{ nullable: true }' : '';
                const tsRequired = column.isNullable || column.default ? '?' : '!';
                const tsType = utils_1.columnToTypeScriptType(column);
                return `
            @FieldGraphQL(() => ${graphQLDataType}, ${nullable})
            ${column.propertyName}${tsRequired}: ${tsType};
            `;
            })
                .join(''),
            enumImports: relation.inverseEntityMetadata.columns
                .filter((column) => !(column.relationMetadata || column.propertyName === inverseSidePropertyPath) &&
                !column.isPrimary &&
                !column.isCreateDate &&
                !column.isGenerated &&
                !column.isUpdateDate &&
                !column.isVersion &&
                column.enum)
                .reduce((prev, column) => {
                const tsType = utils_1.columnToTypeScriptType(column);
                if (!prev.includes(tsType)) {
                    prev = [...prev, tsType];
                }
                return prev;
            }, [])
        });
        const getRelationsWithImports = () => {
            const imports = [];
            const relations = relation.inverseEntityMetadata.relations
                .filter(inverseRelation => inverseRelation.propertyName !== inverseSidePropertyPath)
                .map(inverseRelation => {
                const { inverseGraphqlType } = utils_1.inverseRelationToTypes(inverseRelation);
                const nullable = inverseRelation.isNullable ? '{nullable:true}' : '';
                const tsRequired = inverseRelation.isNullable ? '?' : '!';
                const propertyName = inverseRelation.propertyName;
                const inverseSidePropertyPath = inverseRelation.inverseSidePropertyPath;
                const isInverse = Boolean(inverseRelation.inverseEntityMetadata.relations.find(inverseInverseRelation => inverseInverseRelation.propertyName === inverseSidePropertyPath));
                const inverseSidePropertyPathCapitalized = inverseSidePropertyPath.charAt(0).toUpperCase() + inverseSidePropertyPath.slice(1);
                if (inverseGraphqlType.constructor === graphql_1.GraphQLObjectType) {
                    switch (inverseRelation.relationType) {
                        case 'many-to-one':
                        case 'one-to-one':
                            if (!imports.includes(`${inverseGraphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`)) {
                                imports.push(`${inverseGraphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`);
                            }
                            return `
              @FieldGraphQL(() => ${inverseGraphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input,${nullable})
              ${propertyName}${tsRequired}: ${inverseGraphqlType.name}CreateOne${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input;
              `;
                        case 'one-to-many':
                        case 'many-to-many':
                            if (!imports.includes(`${inverseGraphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`)) {
                                imports.push(`${inverseGraphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input`);
                            }
                            return `
              @FieldGraphQL(() => ${inverseGraphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input,${nullable})
              ${propertyName}${tsRequired}: ${inverseGraphqlType.name}CreateMany${isInverse ? 'Without' + inverseSidePropertyPathCapitalized : ''}Input;
              `;
                    }
                }
                return '';
            });
            return { relations, imports };
        };
        const createInputName = inputName('');
        const createInputOneName = inputName('One');
        const createInputManyName = inputName('Many');
        switch (relation.relationType) {
            case 'many-to-one':
            case 'one-to-one':
                if (!Object.keys(createInputs).includes(createInputName)) {
                    createInputs[createInputName] = {
                        ...getFields(),
                        ...getRelationsWithImports()
                    };
                }
                if (!Object.keys(createInputs).includes(createInputOneName)) {
                    createInputs[createInputOneName] = {
                        fields: `
              @FieldGraphQL(() => ${graphqlType.name}WhereUniqueInput,{nullable:true})
              connect?: ${graphqlType.name}WhereUniqueInput;
              @FieldGraphQL(() => ${createInputName},{nullable:true})
              create?: ${createInputName}
              `,
                        imports: [`${graphqlType.name}WhereUniqueInput`, createInputName]
                    };
                }
                break;
            case 'one-to-many':
            case 'many-to-many':
                if (!Object.keys(createInputs).includes(createInputName)) {
                    createInputs[createInputName] = {
                        ...getFields(),
                        ...getRelationsWithImports()
                    };
                }
                if (!Object.keys(createInputs).includes(createInputManyName)) {
                    createInputs[createInputManyName] = {
                        fields: `
              @FieldGraphQL(() => [${graphqlType.name}WhereUniqueInput],{nullable:true})
              connect?: ${graphqlType.name}WhereUniqueInput[];
              @FieldGraphQL(() => [${createInputName}],{nullable:true})
              create?: ${createInputName}[]
              `,
                        imports: [`${graphqlType.name}WhereUniqueInput`, createInputName]
                    };
                }
                break;
        }
    });
    return Object.keys(createInputs).reduce((prev, createInput) => {
        return {
            ...prev,
            [createInput]: `
        import 'reflect-metadata';
        import { ArgsType, Field as FieldGraphQL, Float, GraphQLISODateTime,
               ID, InputType,
               ObjectType, registerEnumType } from 'type-graphql';
        import { GraphQLJSONObject, IDType, ConnectionArgs, PageInfo,
                 GraphQLBoolean, GraphQLFloat, GraphQLID,
                 GraphQLInt, GraphQLString, ${(createInputs[createInput].enumImports || []).join(', ')} } from '../..';
        ${createInputs[createInput].imports.length
                ? `
        import { ${createInputs[createInput].imports.join(', ')} } from '../..';
        `
                : ''}
        @InputType()
        export class ${createInput} {
        ${createInputs[createInput].fields}
        ${(createInputs[createInput].relations || []).join('')}
      }
      `
        };
    }, {});
}
exports.entityToCreateInputs = entityToCreateInputs;
function entityToCreateManyArgs(entity) {
    return `
    @ArgsType()
    export class ${entity.name}CreateManyArgs {
      @FieldGraphQL(() => [${entity.name}CreateInput])
      data!: ${entity.name}CreateInput[];
    }
  `;
}
exports.entityToCreateManyArgs = entityToCreateManyArgs;
