"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("./utils");
const graphql_1 = require("graphql");
const type_graphql_1 = require("type-graphql");
function entityToWhereInputImporter(entity) {
    return `
    import {${entity.name}WhereInput} from './input/read/${entity.name}WhereInput';
    export {${entity.name}WhereInput} from './input/read/${entity.name}WhereInput';
  `;
}
exports.entityToWhereInputImporter = entityToWhereInputImporter;
function entityToWhereUniqueInput(entity) {
    const uniques = utils_1.uniquesForEntity(entity);
    const numUniques = entity.columns.reduce((num, column) => {
        if (uniques.includes(column.propertyName) || column.isPrimary) {
            num++;
        }
        return num;
    }, 0);
    const uniqueFieldsAreNullable = numUniques > 1;
    let fieldsTemplate = '';
    entity.columns.forEach((column) => {
        if (uniques.includes(column.propertyName) || column.isPrimary) {
            const nullable = uniqueFieldsAreNullable ? ',{nullable:true}' : '';
            const graphQLDataType = utils_1.columnTypeToGraphQLDataType(column);
            const tsType = utils_1.columnToTypeScriptType(column);
            const tsRequired = column.isNullable ? '?' : '!';
            fieldsTemplate += `
        @FieldGraphQL(() => ${graphQLDataType}${nullable})
        ${column.propertyName}${tsRequired}: ${tsType};
      `;
        }
    });
    return `
    @InputType()
    export class ${entity.name}WhereUniqueInput {
      ${fieldsTemplate}
    }
  `;
}
exports.entityToWhereUniqueInput = entityToWhereUniqueInput;
function entityToWhereInput(entity) {
    let fieldTemplates = `
        @FieldGraphQL(() => [${entity.name}WhereInput],{nullable:true})
        AND?: ${entity.name}WhereInput[];
        @FieldGraphQL(() => [${entity.name}WhereInput],{nullable:true})
        OR?: ${entity.name}WhereInput[];
        @FieldGraphQL(() => [${entity.name}WhereInput],{nullable:true})
        NOT?: ${entity.name}WhereInput[];`;
    const whereInputImport = [];
    const enumImports = [];
    entity.columns
        .filter((column) => !column.relationMetadata)
        .forEach((column) => {
        const { graphqlType, tsType } = utils_1.columnToTypes(column);
        const graphQLDataType = utils_1.columnTypeToGraphQLDataType(column);
        if (column.enum && !enumImports.includes(tsType)) {
            enumImports.push(tsType);
        }
        switch (graphqlType) {
            case graphql_1.GraphQLID:
            case graphql_1.GraphQLString:
                fieldTemplates += `
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_not?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_gt?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_gte?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_lt?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_lte?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_contains?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_not_contains?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_starts_with?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_not_starts_with?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_ends_with?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_not_ends_with?: ${tsType};
        @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
        ${column.propertyName}_in?: ${tsType}[];
        @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
        ${column.propertyName}_not_in?: ${tsType}[];`;
                break;
            case graphql_1.GraphQLInt:
            case graphql_1.GraphQLFloat:
            case type_graphql_1.GraphQLISODateTime:
                fieldTemplates += `
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_not?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_gt?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_gte?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_lt?: ${tsType};
        @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
        ${column.propertyName}_lte?: ${tsType};
        @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
        ${column.propertyName}_in?: ${tsType}[];
        @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
        ${column.propertyName}_not_in?: ${tsType}[];`;
                break;
            default:
                fieldTemplates += `
          @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
          ${column.propertyName}?: ${tsType};
          @FieldGraphQL(() => ${graphQLDataType},{nullable:true})
          ${column.propertyName}_not?: ${tsType};
          @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
          ${column.propertyName}_in?: ${tsType}[];
          @FieldGraphQL(() => [${graphQLDataType}],{nullable:true})
          ${column.propertyName}_not_in?: ${tsType}[];`;
                break;
        }
    });
    entity.relations.forEach((relation) => {
        const { graphqlType } = utils_1.relationToTypes(relation);
        if (!relation.inverseEntityMetadata || graphqlType.constructor !== graphql_1.GraphQLObjectType)
            return;
        if (!whereInputImport.includes(`${graphqlType.name}WhereInput`)) {
            whereInputImport.push(`${graphqlType.name}WhereInput`);
        }
        switch (relation.relationType) {
            case 'many-to-one':
            case 'one-to-one':
                fieldTemplates += `
      @FieldGraphQL(() => ${graphqlType.name}WhereInput,{nullable:true})
      ${relation.propertyName}?: ${graphqlType.name}WhereInput;
      `;
                break;
            case 'one-to-many':
            case 'many-to-many':
                fieldTemplates += `
      @FieldGraphQL(() => ${graphqlType.name}WhereInput,{nullable:true})
      ${relation.propertyName}_some?: ${graphqlType.name}WhereInput;
      @FieldGraphQL(() => ${graphqlType.name}WhereInput,{nullable:true})
      ${relation.propertyName}_every?: ${graphqlType.name}WhereInput;
      @FieldGraphQL(() => ${graphqlType.name}WhereInput,{nullable:true})
      ${relation.propertyName}_none?: ${graphqlType.name}WhereInput;
      `;
                break;
        }
    });
    return `
  import 'reflect-metadata';
  import { ArgsType, Field as FieldGraphQL, Float, GraphQLISODateTime,
               ID, InputType,
               ObjectType, registerEnumType } from 'type-graphql';
  import { GraphQLJSONObject, IDType, ConnectionArgs,
           GraphQLBoolean, GraphQLFloat, GraphQLID,
           GraphQLInt, GraphQLString, ${enumImports.join(',')} } from '../..';
  ${whereInputImport.reduce((prev, next) => next !== `${entity.name}WhereInput`
        ? prev +
            `
  import {${next}} from './${next}'
  `
        : prev, '')}
  @InputType()
  export class ${entity.name}WhereInput {
  ${fieldTemplates}
  }
  `;
}
exports.entityToWhereInput = entityToWhereInput;
function entityToOrderByEnum(entity) {
    let fieldsTemplate = '';
    entity.columns
        .filter((col) => !col.relationMetadata)
        .forEach((column) => {
        fieldsTemplate += `
        ${column.propertyName}_ASC = '${column.propertyName}_ASC',
        ${column.propertyName}_DESC = '${column.propertyName}_DESC',
      `;
    });
    return `
    export enum ${entity.name}OrderByEnum {
      ${fieldsTemplate}
    }
    registerEnumType(${entity.name}OrderByEnum, {
      name: '${entity.name}OrderByInput'
    });
  `;
}
exports.entityToOrderByEnum = entityToOrderByEnum;
function entityToWhereArgs(entity) {
    return `
    @ArgsType()
    export class ${entity.name}WhereArgs extends ConnectionArgs {
      @FieldGraphQL(() => ${entity.name}WhereInput,{nullable:true})
      where?: ${entity.name}WhereInput;
      @FieldGraphQL(() => [${entity.name}OrderByEnum],{nullable:true})
      orderBy?: ${entity.name}OrderByEnum[];
    }
  `;
}
exports.entityToWhereArgs = entityToWhereArgs;
function entityToConnectionResponse(entity) {
    return `
    @ObjectType()
    export class ${entity.name}Edge {
      @FieldGraphQL()
      cursor: string;
      @FieldGraphQL(() => ${entity.name})
      node: ${entity.name};
    }
    @ObjectType()
    export class Aggregate${entity.name} {
      @FieldGraphQL()
      count: number
    }
    @ObjectType()
    export class ${entity.name}Connection {
      @FieldGraphQL(() => [${entity.name}Edge])
      edges: ${entity.name}Edge[]
      @FieldGraphQL()
      pageInfo: PageInfo
      @FieldGraphQL()
      aggregate: Aggregate${entity.name}
    }
  `;
}
exports.entityToConnectionResponse = entityToConnectionResponse;
